@@ -33,6 +33,7 @@ import java.util.Set;
 import java.util.StringTokenizer;
 
 import org.apache.openjpa.jdbc.identifier.DBIdentifier;
+import org.apache.openjpa.jdbc.identifier.QualifiedDBIdentifier;
 import org.apache.openjpa.jdbc.kernel.JDBCFetchConfiguration;
 import org.apache.openjpa.jdbc.kernel.exps.FilterValue;
 import org.apache.openjpa.jdbc.kernel.exps.Lit;
@@ -83,6 +84,9 @@ public class DB2Dictionary
     protected static final String useKeepExclusiveLockClause
         = "USE AND KEEP EXCLUSIVE LOCKS";
     protected static final String forReadOnlyClause = "FOR READ ONLY";
+    protected static final String defaultSequenceSQL 
+        = "SELECT SEQSCHEMA AS SEQUENCE_SCHEMA, SEQNAME AS SEQUENCE_NAME FROM SYSCAT.SEQUENCES";
+
     protected String databaseProductName = "";
     protected String databaseProductVersion = "";
     protected int maj = 0;
@@ -98,8 +102,7 @@ public class DB2Dictionary
 
         nextSequenceQuery = "VALUES NEXTVAL FOR {0}";
 
-        sequenceSQL = "SELECT SEQSCHEMA AS SEQUENCE_SCHEMA, "
-            + "SEQNAME AS SEQUENCE_NAME FROM SYSCAT.SEQUENCES";
+        sequenceSQL = defaultSequenceSQL;
         sequenceSchemaSQL = "SEQSCHEMA = ?";
         sequenceNameSQL = "SEQNAME = ?";
         characterColumnSize = 254;
@@ -354,8 +357,14 @@ public class DB2Dictionary
                 + "SYSIBM.SYSDUMMY1";
             nextSequenceQuery = "SELECT NEXTVAL FOR {0} FROM "
                 + "SYSIBM.SYSDUMMY1";
-            sequenceSQL = "SELECT SCHEMA AS SEQUENCE_SCHEMA, "
-                + "NAME AS SEQUENCE_NAME FROM SYSIBM.SYSSEQUENCES";
+            // allow users to set a non default sequenceSQL. 
+            if (defaultSequenceSQL.equals(sequenceSQL)){            	
+	            sequenceSQL = "SELECT SCHEMA AS SEQUENCE_SCHEMA, "
+	                + "NAME AS SEQUENCE_NAME FROM SYSIBM.SYSSEQUENCES";
+	            
+                if (log.isTraceEnabled())
+                    log.trace(_loc.get("sequencesql-override", new Object[] {defaultSequenceSQL, sequenceSQL}));
+            }
             sequenceSchemaSQL = "SCHEMA = ?";
             sequenceNameSQL = "NAME = ?";
             if (maj == 8) {
@@ -372,8 +381,14 @@ public class DB2Dictionary
                 + "SYSIBM.SYSDUMMY1";
             validationSQL = "SELECT DISTINCT(CURRENT TIMESTAMP) FROM "
                 + "QSYS2.SYSTABLES";
-            sequenceSQL = "SELECT SEQUENCE_SCHEMA, "
-                + "SEQUENCE_NAME FROM QSYS2.SYSSEQUENCES";
+            // allow users to set a non default sequenceSQL.
+            if (defaultSequenceSQL.equals(sequenceSQL)){            
+	            sequenceSQL = "SELECT SEQUENCE_SCHEMA, "
+	                + "SEQUENCE_NAME FROM QSYS2.SYSSEQUENCES";
+	            
+                if (log.isTraceEnabled())
+                    log.trace(_loc.get("sequencesql-override", new Object[] {defaultSequenceSQL, sequenceSQL}));
+            }
             sequenceSchemaSQL = "SEQUENCE_SCHEMA = ?";
             sequenceNameSQL = "SEQUENCE_NAME = ?";
             break;
@@ -912,8 +927,10 @@ public class DB2Dictionary
             // build the index for the sequence tables
             // the index name will be the fully qualified table name + _IDX
             Table tab = schema.getTable(table);
-            DBIdentifier idxName = DBIdentifier.append(tab.getFullIdentifier(), "IDX");
-            Index idx = tab.addIndex(getValidIndexName(idxName, tab));
+            DBIdentifier fullIdxId = tab.getFullIdentifier().clone();
+            DBIdentifier unQualifiedName = DBIdentifier.append(fullIdxId.getUnqualifiedName(), "IDX");
+            fullIdxId.setName(getValidIndexName(unQualifiedName, tab));
+            Index idx = tab.addIndex(fullIdxId);
             idx.setUnique(true);
             idx.addColumn(pkColumn);
         }
